# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.close_o_auth_success_response import CloseOAuthSuccessResponse
from .raw_client import AsyncRawCloseOauthClient, RawCloseOauthClient


class CloseOauthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCloseOauthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCloseOauthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCloseOauthClient
        """
        return self._raw_client

    def authorize_close(
        self,
        *,
        instance_id: str,
        client_id: typing.Optional[str] = None,
        scope: typing.Optional[str] = None,
        redirect_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Start Close OAuth flow

        Parameters:
        - instance_id: Identifier for the instance requesting authorization
        - client_id: Optional client ID for white labeling
        - scope: Optional scopes to request (comma-separated)
        - redirect_url: Optional URL to redirect to after authorization completes

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting authorization

        client_id : typing.Optional[str]
            Client ID for white labeling, if not provided will use default credentials

        scope : typing.Optional[str]
            Optional OAuth scopes to request (comma-separated string)

        redirect_url : typing.Optional[str]
            Optional URL to redirect to after authorization completes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.close_oauth.authorize_close(
            instance_id="instance_id",
        )
        """
        _response = self._raw_client.authorize_close(
            instance_id=instance_id,
            client_id=client_id,
            scope=scope,
            redirect_url=redirect_url,
            request_options=request_options,
        )
        return _response.data

    def close_o_auth_callback(
        self,
        *,
        code: typing.Optional[str] = None,
        state: typing.Optional[str] = None,
        error: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CloseOAuthSuccessResponse:
        """
        Handles the callback from Close OAuth authorization.

        Parameters
        ----------
        code : typing.Optional[str]
            Authorization code returned by Close

        state : typing.Optional[str]
            State parameter containing encoded authorization data

        error : typing.Optional[str]
            Error code returned by Close, if any

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloseOAuthSuccessResponse
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.close_oauth.close_o_auth_callback()
        """
        _response = self._raw_client.close_o_auth_callback(
            code=code, state=state, error=error, request_options=request_options
        )
        return _response.data

    def refresh_close_token(
        self, *, instance_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CloseOAuthSuccessResponse:
        """
        Refresh an expired Close access token using the stored refresh token

        Parameters:
        - instance_id: Identifier for the instance requesting token refresh

        Returns:
        - Success response if token was refreshed successfully, error response otherwise

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting token refresh

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloseOAuthSuccessResponse
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.close_oauth.refresh_close_token(
            instance_id="instance_id",
        )
        """
        _response = self._raw_client.refresh_close_token(instance_id=instance_id, request_options=request_options)
        return _response.data

    def revoke_close_token(
        self, *, instance_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CloseOAuthSuccessResponse:
        """
        Revoke Close access and refresh tokens

        Parameters:
        - instance_id: Identifier for the instance requesting token revocation

        Returns:
        - Success response if token was revoked successfully, error response otherwise

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting token revocation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloseOAuthSuccessResponse
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.close_oauth.revoke_close_token(
            instance_id="instance_id",
        )
        """
        _response = self._raw_client.revoke_close_token(instance_id=instance_id, request_options=request_options)
        return _response.data


class AsyncCloseOauthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCloseOauthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCloseOauthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCloseOauthClient
        """
        return self._raw_client

    async def authorize_close(
        self,
        *,
        instance_id: str,
        client_id: typing.Optional[str] = None,
        scope: typing.Optional[str] = None,
        redirect_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Start Close OAuth flow

        Parameters:
        - instance_id: Identifier for the instance requesting authorization
        - client_id: Optional client ID for white labeling
        - scope: Optional scopes to request (comma-separated)
        - redirect_url: Optional URL to redirect to after authorization completes

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting authorization

        client_id : typing.Optional[str]
            Client ID for white labeling, if not provided will use default credentials

        scope : typing.Optional[str]
            Optional OAuth scopes to request (comma-separated string)

        redirect_url : typing.Optional[str]
            Optional URL to redirect to after authorization completes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.close_oauth.authorize_close(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.authorize_close(
            instance_id=instance_id,
            client_id=client_id,
            scope=scope,
            redirect_url=redirect_url,
            request_options=request_options,
        )
        return _response.data

    async def close_o_auth_callback(
        self,
        *,
        code: typing.Optional[str] = None,
        state: typing.Optional[str] = None,
        error: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CloseOAuthSuccessResponse:
        """
        Handles the callback from Close OAuth authorization.

        Parameters
        ----------
        code : typing.Optional[str]
            Authorization code returned by Close

        state : typing.Optional[str]
            State parameter containing encoded authorization data

        error : typing.Optional[str]
            Error code returned by Close, if any

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloseOAuthSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.close_oauth.close_o_auth_callback()


        asyncio.run(main())
        """
        _response = await self._raw_client.close_o_auth_callback(
            code=code, state=state, error=error, request_options=request_options
        )
        return _response.data

    async def refresh_close_token(
        self, *, instance_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CloseOAuthSuccessResponse:
        """
        Refresh an expired Close access token using the stored refresh token

        Parameters:
        - instance_id: Identifier for the instance requesting token refresh

        Returns:
        - Success response if token was refreshed successfully, error response otherwise

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting token refresh

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloseOAuthSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.close_oauth.refresh_close_token(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.refresh_close_token(instance_id=instance_id, request_options=request_options)
        return _response.data

    async def revoke_close_token(
        self, *, instance_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CloseOAuthSuccessResponse:
        """
        Revoke Close access and refresh tokens

        Parameters:
        - instance_id: Identifier for the instance requesting token revocation

        Returns:
        - Success response if token was revoked successfully, error response otherwise

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting token revocation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CloseOAuthSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.close_oauth.revoke_close_token(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke_close_token(instance_id=instance_id, request_options=request_options)
        return _response.data
