# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.g_drive_o_auth_success_response import GDriveOAuthSuccessResponse
from .raw_client import AsyncRawGdriveOauthClient, RawGdriveOauthClient


class GdriveOauthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGdriveOauthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGdriveOauthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGdriveOauthClient
        """
        return self._raw_client

    def authorize_g_drive(
        self,
        *,
        instance_id: str,
        client_id: typing.Optional[str] = None,
        scope: typing.Optional[str] = None,
        redirect_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Start Google Drive OAuth flow

        Parameters:
        - instance_id: Identifier for the instance requesting authorization
        - client_id: Optional client ID for white labeling
        - scope: Optional scopes to request (comma-separated)
        - redirect_url: Optional URL to redirect to after authorization completes

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting authorization

        client_id : typing.Optional[str]
            Client ID for white labeling, if not provided will use default credentials

        scope : typing.Optional[str]
            Optional OAuth scopes to request (comma-separated string)

        redirect_url : typing.Optional[str]
            Optional URL to redirect to after authorization completes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.gdrive_oauth.authorize_g_drive(
            instance_id="instance_id",
        )
        """
        _response = self._raw_client.authorize_g_drive(
            instance_id=instance_id,
            client_id=client_id,
            scope=scope,
            redirect_url=redirect_url,
            request_options=request_options,
        )
        return _response.data

    def gdrive_o_auth_callback(
        self,
        *,
        code: typing.Optional[str] = None,
        state: typing.Optional[str] = None,
        error: typing.Optional[str] = None,
        error_description: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GDriveOAuthSuccessResponse:
        """
        Handles the callback from Google Drive OAuth authorization.

        Parameters
        ----------
        code : typing.Optional[str]
            Authorization code returned by Google Drive

        state : typing.Optional[str]
            State parameter containing encoded authorization data

        error : typing.Optional[str]
            Error code returned by Google Drive, if any

        error_description : typing.Optional[str]
            Detailed error description from Google Drive, if any

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GDriveOAuthSuccessResponse
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.gdrive_oauth.gdrive_o_auth_callback()
        """
        _response = self._raw_client.gdrive_o_auth_callback(
            code=code, state=state, error=error, error_description=error_description, request_options=request_options
        )
        return _response.data

    def refresh_gdrive_token(
        self, *, instance_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GDriveOAuthSuccessResponse:
        """
        Refresh an expired Google Drive access token using the stored refresh token

        Parameters:
        - instance_id: Identifier for the instance requesting token refresh

        Returns:
        - Success response if token was refreshed successfully, error response otherwise

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting token refresh

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GDriveOAuthSuccessResponse
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.gdrive_oauth.refresh_gdrive_token(
            instance_id="instance_id",
        )
        """
        _response = self._raw_client.refresh_gdrive_token(instance_id=instance_id, request_options=request_options)
        return _response.data

    def gdrive_picker_page(
        self,
        *,
        client_id: typing.Optional[str] = None,
        api_key: typing.Optional[str] = None,
        scope: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Serve the Google Drive picker page with the appropriate credentials

        Parameters
        ----------
        client_id : typing.Optional[str]
            Client ID to use for the picker

        api_key : typing.Optional[str]
            API key to use for the picker

        scope : typing.Optional[str]
            OAuth scopes for the picker

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        from klavis import Klavis

        client = Klavis(
            api_key="YOUR_API_KEY",
        )
        client.gdrive_oauth.gdrive_picker_page()
        """
        _response = self._raw_client.gdrive_picker_page(
            client_id=client_id, api_key=api_key, scope=scope, request_options=request_options
        )
        return _response.data


class AsyncGdriveOauthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGdriveOauthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGdriveOauthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGdriveOauthClient
        """
        return self._raw_client

    async def authorize_g_drive(
        self,
        *,
        instance_id: str,
        client_id: typing.Optional[str] = None,
        scope: typing.Optional[str] = None,
        redirect_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Start Google Drive OAuth flow

        Parameters:
        - instance_id: Identifier for the instance requesting authorization
        - client_id: Optional client ID for white labeling
        - scope: Optional scopes to request (comma-separated)
        - redirect_url: Optional URL to redirect to after authorization completes

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting authorization

        client_id : typing.Optional[str]
            Client ID for white labeling, if not provided will use default credentials

        scope : typing.Optional[str]
            Optional OAuth scopes to request (comma-separated string)

        redirect_url : typing.Optional[str]
            Optional URL to redirect to after authorization completes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.gdrive_oauth.authorize_g_drive(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.authorize_g_drive(
            instance_id=instance_id,
            client_id=client_id,
            scope=scope,
            redirect_url=redirect_url,
            request_options=request_options,
        )
        return _response.data

    async def gdrive_o_auth_callback(
        self,
        *,
        code: typing.Optional[str] = None,
        state: typing.Optional[str] = None,
        error: typing.Optional[str] = None,
        error_description: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GDriveOAuthSuccessResponse:
        """
        Handles the callback from Google Drive OAuth authorization.

        Parameters
        ----------
        code : typing.Optional[str]
            Authorization code returned by Google Drive

        state : typing.Optional[str]
            State parameter containing encoded authorization data

        error : typing.Optional[str]
            Error code returned by Google Drive, if any

        error_description : typing.Optional[str]
            Detailed error description from Google Drive, if any

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GDriveOAuthSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.gdrive_oauth.gdrive_o_auth_callback()


        asyncio.run(main())
        """
        _response = await self._raw_client.gdrive_o_auth_callback(
            code=code, state=state, error=error, error_description=error_description, request_options=request_options
        )
        return _response.data

    async def refresh_gdrive_token(
        self, *, instance_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GDriveOAuthSuccessResponse:
        """
        Refresh an expired Google Drive access token using the stored refresh token

        Parameters:
        - instance_id: Identifier for the instance requesting token refresh

        Returns:
        - Success response if token was refreshed successfully, error response otherwise

        Parameters
        ----------
        instance_id : str
            Unique identifier for the client instance requesting token refresh

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GDriveOAuthSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.gdrive_oauth.refresh_gdrive_token(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.refresh_gdrive_token(
            instance_id=instance_id, request_options=request_options
        )
        return _response.data

    async def gdrive_picker_page(
        self,
        *,
        client_id: typing.Optional[str] = None,
        api_key: typing.Optional[str] = None,
        scope: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Optional[typing.Any]:
        """
        Serve the Google Drive picker page with the appropriate credentials

        Parameters
        ----------
        client_id : typing.Optional[str]
            Client ID to use for the picker

        api_key : typing.Optional[str]
            API key to use for the picker

        scope : typing.Optional[str]
            OAuth scopes for the picker

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Optional[typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from klavis import AsyncKlavis

        client = AsyncKlavis(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.gdrive_oauth.gdrive_picker_page()


        asyncio.run(main())
        """
        _response = await self._raw_client.gdrive_picker_page(
            client_id=client_id, api_key=api_key, scope=scope, request_options=request_options
        )
        return _response.data
